classdef ImplicitGSIPProblem<GSIPProblem
    %ImlicitGSIPProblem is an abstract class. It is the super class for all
    %GSIP's whose index set Y(x) is given by implict constraints. It
    %contains implementations of all functions. These functions are used if
    %no function is given in a particular instance. 
   
    
    properties (Abstract, GetAccess=protected)
        nrOfLowLevNonlinConstrs
    end
    
    
    methods (Abstract)     
        % y0=getInitValsLowLevVars(j,x) returns an interior point y0 of the
        %  j-th index-set. These point can be used as an starting point for
        %  the optimization of the lower level problem.
                          

    end
    
    methods %set standard values
        %sets the properties
        function setStandardValuesForProperties(obj)
            obj.boundsOnUppLevVars=[];
            obj.initValsOfUppLevVars=[];
            obj.nrOfNonlinIneqsUppLev=[];   
            obj.nrOfNonlinEqsUppLev=[];    
                          
            obj.linIneqsOnUppLevVars=[];   
            obj.linEqsOnUppLevVars=[];      
            obj.nrOfIndexset=[];            
            obj.con2LowLev=[];
            obj.nrOfLowLevNonlinConstrs=[];
            
        end
        %sets the dependend properties
        function setDependendValuesForProperties(obj)
            if isempty(obj.boundsOnUppLevVars)
                obj.boundsOnUppLevVars=[-inf*ones(1,obj.nrOfUppLevVars);inf*ones(1,obj.nrOfUppLevVars)];
            end
            if isempty(obj.initValsOfUppLevVars)
                obj.initValsOfUppLevVars=zeros(1,obj.nrOfUppLevVars);
            end
            if isempty(obj.nrOfNonlinIneqsUppLev)
                try
                    g=obj.evalUppLevNonlinIneqConstrs(obj.initValsOfUppLevVars);
                    obj.nrOfNonlinIneqsUppLev=size(g,2);
                catch me
                    warning('Wasn''t able to execute the function ''evalUppLevNonlinIneqConstrs''. Some properties might be wrong. Please set the property ''nrOfNonlinIneqsUppLev''.') 
                end
            end
            if isempty(obj.nrOfNonlinEqsUppLev)
                try
                    h=obj.evalUppLevNonlinEqConstrs(obj.initValsOfUppLevVars);
                    obj.nrOfNonlinEqsUppLev=size(h,2);
                catch me
                    warning('Wasn''t able to execute the function ''evalUppLevNonlinEqConstrs''. Some properties might be wrong. Please set the property ''nrOfNonlinEqsUppLev''.') 
                end         
            end
            if isempty(obj.nrOfIndexset)
                obj.nrOfIndexset=1;                
            end
            if isempty(obj.con2LowLev)
                obj.con2LowLev=ones(1,obj.nrOfSemiInfConstrs);                
            end
            
            if isempty(obj.nrOfLowLevNonlinConstrs)
                try
                    obj.nrOfLowLevNonlinConstrs=zeros(1,obj.nrOfIndexset);
                    for i=1:obj.nrOfIndexset
                        u=obj.evalLowLevNonlinConstrs(i,obj.initValsOfUppLevVars,obj.getInitValsLowLevVars(i,obj.initValsOfUppLevVars));
                        obj.nrOfLowLevNonlinConstrs(i)=size(u,2);
                    end
                catch me
                    warning('Wasn''t able to execute the function ''evalLowLevNonlinConstrs''. The properties might be wrong. Please set the property ''nrOfLowLevNonlinConstrs''.') 
                end    
            end
        end
    end
    methods %can be overwritten
        %bounds on lower-level variables
        function [lb,ub]=getBoundsOnLowLevVars(obj,j,x)
            %[lb,ub]=getBoundsOnLowLevVars(j,x) returns a bounding box
            %of the j-th index set for a fixed point x. lb and ub are
            %rowvectors. If one variable yi is unbounded, the bounds are
            %-inf/inf.
            lb=-inf*ones(1,obj.nrOfLowLevVars(j));
            ub=inf*ones(1,obj.nrOfLowLevVars(j));
        end
        %Derivativs
        function [Dxlb,Dxub]=DxBoundsOnLowLevVars(obj,j,x0)
            %[Dxlb,Dxub]=DxBoundsOnLowLevVars(j,x) returns the
            %derivative with respect to x (as Jacobian) of the function 
            %[lb,ub]=getBoundsOnLowLevVars(j,x). If no function is
            %implemented, a numerical solution will be computed.
            [Dxlb,Dxub]=getDerivativeViaDifferenceQuotient(@(x) obj.getBoundsOnLowLevVars(j,x),x0,{obj.nrOfLowLevVars(j),obj.nrOfLowLevVars(j)});
        end
        function [DxDxlb,DxDxub]=DxDxBoundsOnLowLevVars(obj,j,x0)
            %[DxDxlb,DxDxub]=DxDxBoundsOnLowLevVars(j,x) returns the
            %seccond derivative with respect to x of the function 
            %[lb,ub]=getBoundsOnLowLevVars(j,x). If no function is
            %implemented, a numerical solution will be computed. For the
            %exact output size have a look into the documentation.
            [DxDxlb,DxDxub]=getSecDerivativeViaDifferenceQuotient(@(x) obj.getBoundsOnLowLevVars(j,x),x0,{[obj.nrOfLowLevVars(j)],[obj.nrOfLowLevVars(j)]});
        end
        
        
        
        %Linear inequalities
        function [A,b]=getLinIneqsOnLowLevVars(obj,j,x)
            %[A,b]=getLinIneqsOnLowLevVars(j,x) returns the linear
            %inequality constraints of the j-th index set A*y<=b.
            A=zeros(0,obj.nrOfLowLevVars(j));
            b=zeros(0,1);
        end
        %Derivatives
        function [DxA,Dxb]=DxLinIneqsOnLowLevVars(obj,j,x0)
            %[DxA,Dxb]=DxLinIneqsOnLowLevVars(j,x) returns the
            %derivative with respect to x of the function 
            %[A,b]=getLinIneqsOnLowLevVars(j,x). If no function is
            %implemented, a numerical solution will be computed. Dxb is the
            %Jacobian. DxA is a 3-dimensional matrix where the first matrix
            %is the derivative with respect to x1 the seccond with respect
            %to x2 and so on.
            nrLinIneqs=size(obj.getLinIneqsOnLowLevVars(j,x0),1);
            [DxA,Dxb]=getDerivativeViaDifferenceQuotient(@(x) obj.getLinIneqsOnLowLevVars(j,x),x0,{[nrLinIneqs,obj.nrOfLowLevVars(j)],[nrLinIneqs,1]});
        end
        function [DxDxA,DxDxb]=DxDxLinIneqsOnLowLevVars(obj,j,x0)
            %[DxDxA,DxDxb]=DxDxLinIneqsOnLowLevVars(j,x) returns the
            %seccond derivative with respect to x of the function 
            %[A,b]=getLinIneqsOnLowLevVars(j,x). If no function is
            %implemented, a numerical solution will be computed. For the
            %exact output size have a look into the documentation.
            nrLinIneqs=size(obj.getLinIneqsOnLowLevVars(j,x0),1);
            [DxDxA,DxDxb]=getSecDerivativeViaDifferenceQuotient(@(x) obj.getLinIneqsOnLowLevVars(j,x),x0,{[nrLinIneqs,obj.nrOfLowLevVars(j)],[nrLinIneqs,1]});            
        end
        
        
        
        function u=evalLowLevNonlinConstrs(obj,i,x,y) 
            %u=evalLowLevNonlinConstrs(j,x,y) evaluates the non linear 
            %constraint of the j-th index set. The output u is a vector
            %where every entry corresponds to one constraint. If no such
            %constraints are implemented, a empty vector of size 1x0 is
            %returned.
            u=zeros(1,0);
        end
        %Derivatives
        function Dy_u=DyLowLevNonlinConstrs(obj,i,x,y0)
            %Dy_u=DyLowLevNonlinConstrs(j,x,y) returns the derivative (As 
            %Jacobian) of the non linear constraints of the j-th index set
            %with respect to y. If no function is implemented a numerical
            %solution will be computed.
            nrLowLevConstr=size(obj.evalLowLevNonlinConstrs(i,x,y0),2);
            Dy_u=getDerivativeViaDifferenceQuotient(@(y) obj.evalLowLevNonlinConstrs(i,x,y),y0,{nrLowLevConstr});
        end
        function Dx_u=DxLowLevNonlinConstrs(obj,i,x0,y)
            %Dx_u=DxLowLevNonlinConstrs(j,x,y) returns the derivative (As 
            %Jacobian) of the non linear constraints of the j-th index set
            %with respect to x. If no function is implemented, a numerical
            %solution will be computed.
            nrLowLevConstr=size(obj.evalLowLevNonlinConstrs(i,x0,y),2);
            Dx_u=getDerivativeViaDifferenceQuotient(@(x) obj.evalLowLevNonlinConstrs(i,x,y),x0,{nrLowLevConstr});
        end
        
        function DxDx_u=DxDxLowLevNonlinConstrs(obj,i,x0,y)
            %DxDx_u=DxDxLowLevNonlinConstrs(j,x,y) returns the seccond
            %derivative of the non linear constraints of the j-th index
            %set with respect to x. If no function is implemented, a
            %numerical solution will be computed. For the exact output size
            %have a look into the documentation.
            nrLowLevConstr=size(obj.evalLowLevNonlinConstrs(i,x0,y),2);
            DxDx_u=getSecDerivativeViaDifferenceQuotient(@(x) obj.evalLowLevNonlinConstrs(i,x,y),x0,{[nrLowLevConstr]});                     
        end
        
        function DxDy_u=DxDyLowLevNonlinConstrs(obj,i,x0,y0)
            %DxDy_u=DxDyLowLevNonlinConstrs(j,x,y) returns the mixed seccond
            %derivative of the non linear constraints of the j-th index
            %set with respect to x and y. If no function is implemented, a
            %numerical solution will be computed. For the exact output size
            %have a look into the documentation.
            nrLowLevConstr=size(obj.evalLowLevNonlinConstrs(i,x0,y0),2);
            DxDy_u=getMixedSecDerivativeViaDifferenceQuotient(@(y,x) obj.evalLowLevNonlinConstrs(i,x,y),y0,x0,{[nrLowLevConstr]});                     
        end
        
        function DyDy_u=DyDyLowLevNonlinConstrs(obj,i,x,y0)
            %DyDy_u=DyDyLowLevNonlinConstrs(j,x,y) returns the seccond
            %derivative of the non linear constraints of the j-th index
            %set with respect to y. If no function is implemented, a
            %numerical solution will be computed. For the exact output size
            %have a look into the documentation.
            nrLowLevConstr=size(obj.evalLowLevNonlinConstrs(i,x,y0),2);
            DyDy_u=getSecDerivativeViaDifferenceQuotient(@(y) obj.evalLowLevNonlinConstrs(i,x,y),y0,{[nrLowLevConstr]});
        end
        
        function y=getInitValsLowLevVars(obj,i,x)
            y=zeros(1,obj.nrOfLowLevVars(i));
        end
        
        function y=evalTrafoFun(obj,k,x,z)
            y=z;
        end
                
        function z=evalInvTrafoFun(obj,k,x,y)
            z=y;
        end
        
    end
        
    %Get-functions
    methods 
        function n=getNrNonlinLowLevConstrs(obj,j)
            %n=getNrNonlinLowLevConstrs(obj,j) returns the number of non
            %linear constraints on the j-th index set
            n=obj.nrOfLowLevNonlinConstrs(j);                   
        end
        function n=getNrAllLowLevConstrs(obj,j)
            %n=getNrAllLowLevConstrs(obj,j) returns the number of all
            %constraints on the j-th index-set, i. e. non linear
            %constraints, linear constraints and bounds. For the bounds
            %-inf and inf is not taken into account.
            n1=obj.nrOfLowLevNonlinConstrs(j);
            x=obj.initValsOfUppLevVars;
            [A,~]=obj.getLinIneqsOnLowLevVars(j,x);
            n2=size(A,1);
            y=obj.getInitValsLowLevVars(j,x);
            [lb,ub]=obj.getBoundsOnLowLevVars(j,x);
            u3=lb-y;
            n3=sum(u3~=-inf);
            u4=y-ub;
            n4=sum(u4~=-inf);
            n=n1+n2+n3+n4;
        end
    end
    
    %plot Function
    methods
        function []=plotSols(obj,x0,varargin)
            %plotSols(x,points1,color1,points2,color2,...) plots the
            %problem for the parameter x. Moreover, the
            %points are plotted in the specified color.
            
            warning('No routine plotSols is implemented. Try to use the implicit plot function.')
            try
               if max(obj.nrOfLowLevVars) <4                   
                   [lb,ub]=obj.getBoundsOnLowLevVars(1,x0);
                   y0=obj.getInitValsLowLevVars(1,x0);
                   lb(lb==-inf)=y0(lb==-inf)-5;
                   ub(ub==inf)=y0(ub==inf)+5;
                   obj.plotGSIPContainmentCondition(1:obj.nrOfIndexset,x0,lb,ub,varargin{:},'filled');
               else
                   warning('failed to plot the problem. There is no possibility to plot the problem in nrOfLowLevVarsension higher than 3.')
               end
            catch me
               warning('failed to plot the problem.')
            end
        end
    end
    
    
    
    
    %General Transformation   
    methods
        function y=evalGeneralTrafoFun(obj,j,x,z)
                %y=evalGeneralTrafoFun(j,x,z) evaluates a general
                %transformation for the j-th index set. 
                %Idea: y=c(x)+t*r(x,b)*b where c(x) is a point in the interior 
                %of the index set and r(x,b) is the distance from c(x) to the 
                %boundary in direction b.
                %This only works if the indexset is convex or the index set is
                %star shaped and from c(x) all points are reachable with a
                %straight line
                yint=obj.getInitValsLowLevVars(j,x); %interior point c(x)
                m=size(z,1);
                n=size(yint,2);
                y=zeros(m,n);
                for i=1:m
                    %z(i,2:end) is the direction. obj.distBoundary is the
                    %distance r(x,b) to the boundary.
                    y(i,:)=yint+z(i,1)*obj.distBoundary(j,x,z(i,2:end))*z(i,2:end);
                end
            end
        function z=evalInvGeneralTrafoFun(obj,j,x,y)
                %z=evalInvGeneralTrafoFun(j,x,y) returns a preimage z of the
                %general transformation of the point y.
                yint=obj.getInitValsLowLevVars(j,x); %c(x)
                m=size(y,1);
                n=size(yint,2);
                z=zeros(m,n+1);
                for i=1:m
                    temp1=(y(i,:)-yint);
                    ntemp=norm(temp1);%the direction b not normalized
                    if(ntemp==0)    %(c(x) = y)
                        z(i,1)=0;
                        z(i,2)=1;
                        z(i,3:end)=zeros(1,n-1);
                    else            %inverting the upper transformation
                        lambda=obj.distBoundary(j,x,temp1/ntemp); %distance to the boundary r(x,b)
                        z(i,1)=ntemp/lambda;
                        z(i,2:end)=temp1/ntemp; %normalized direction b
                    end
                end                    
            end

         function Dx_y=DxGeneralTrafoFun(obj,j,x,z)
                 %Dx_y=DxGeneralTrafoFun(j,x,z) returns the derivative of
                 %the general transformation with respect to x. The derivative 
                 %is computed with the help of the implicit function theorem.

                 %If the implemented derivative does not work this code can be
                 %used. But the implemented code is much faster.
    %             n=size(z,1);
    %             nrOfLowLevVars=size(z,2)-1;
    %             nrOfUppLevVars=size(x,2);
    %             Dx_y=zeros(nrOfLowLevVars,nrOfUppLevVars,n);
    % 
    %             for i=1:n
    %                 for j=1:nrOfUppLevVars
    %                     x1=x;
    %                     x2=x;
    %                     x1(j)=x1(j)+1.e-9;
    %                     x2(j)=x2(j)-1.e-9;
    %                     Dx_y(:,j,i)=(obj.evalGeneralTrafoFun(x1,z(i,:))-obj.evalGeneralTrafoFun(x2,z(i,:)))/2.e-9;
    %                 end
    %             end

                yint=obj.getInitValsLowLevVars(j,x);
                m=size(z,1);    
                n=size(yint,2);
                k=size(x,2);
                Dx_y=zeros(n,k,m);
                n=obj.getNrNonlinLowLevConstrs(j);
                dc_dx=obj.DxStartingPointLowLev(j,x);            
                for i=1:m       %implicit function theorem.
                     if abs(z(i,1))<1.e-9

                     else
                        r=obj.distBoundary(j,x,z(i,2:end));
                    interP=yint+r*z(i,2:end);

                    u=obj.evalLowLevNonlinConstrs(j,x,interP);
                    temp=u>-5*1.e-2;
                    t=sum(temp);
                    for k=1:n
                        if temp(k)==1
                            temp1=obj.DxLowLevNonlinConstrs(j,x,interP);
                            temp2=obj.DyLowLevNonlinConstrs(j,x,interP);
                            df_dx=temp1(k,:)+temp2(k,:)*dc_dx;
                            df_dl=temp2(k,:)*z(i,2:end)';
                            dphi_dx=-df_dx/df_dl;
                            Dx_y(:,:,i)=Dx_y(:,:,i)+(dc_dx+z(i,1)*z(i,2:end)'*dphi_dx)/t;
                        end
                    end
                    end

                end
            end
    end

    methods (Access=protected)
        function r=distBoundary(obj,j,x,b) %Determine lambda such that u(x,c(x)+lambda*b)=0;
            yint=obj.getInitValsLowLevVars(j,x);
            n=obj.getNrNonlinLowLevConstrs(j);
            if n==1
                 [lambda]=fzero(@(l) obj.zerofun(j,l,1,yint,x,b),0);
                 r=lambda;
                 j=1;
            else
                ID=1;
                crit=true;
                step=1;
                j0=zeros(1,n);
                while crit %search the functions that could be needed for the boundary
                    for i=1:n
                        if obj.zerofun(j,step,i,yint,x,b)>0 %change of sign
                            crit=false;
                            j0(ID)=i;
                            ID=ID+1;
                            
                        end
                    end
                    step=step+1;
                    if(step>1000)
                        error('Did not find a Point outside of the index set in some direction')
                    end
                end
                
                lambda0=zeros(1,ID-1);

                j0=j0(1:ID-1);
                
                ID=1;

                for i=j0 %Intersecting point for all the functions determined above
                    [lambda]=fzero(@(l) obj.zerofun(j,l,i,yint,x,b),step-1.5);

                    lambda0(ID)=lambda;
                    j0(ID)=i;
                    ID=ID+1;
                end
                [r,k]=min(lambda0);%tale the minimal one
                temp=(lambda0-r)<1;
                j=zeros(1,sum(temp));
                ID2=1;
                for i=1:ID-1
                    if temp(i)==1
                        j(ID2)=j0(i);
                        ID2=ID2+1;
                    else
                    end
                end
            end
            
            
        end
        
        
        function f=zerofun(obj,j,lambda,i,yint,x,b)
            %We want lambda to be allways positive
            if lambda <= 0
                temp=obj.evalLowLevNonlinConstrs(j,x,yint);
                f=temp(i);
            else 
                temp=obj.evalLowLevNonlinConstrs(j,x,yint+lambda*b);
                f=temp(i);
            end
        end
    end
    
end

