classdef DesignCircle <Design
    %D=DesignCircle creates an instance of a design D(x) given by:
    %
    %       D(x)={ y in R^2 with:
    %               (y1-x1)^2 + (y2-x2)^2 - x3^2<= 0}
    %       
    %       Vol(x)=pi*x3^2
    %
    %       x in X = { x in R^3 with
    %               10^-6 <= x3}

       
    properties (GetAccess=protected)
        
        dim =2;                             
        nrOfParams=3;                       
        nrOfDesignConstrs=1;                 
        initValsParams=[0,-1,1];      
        
        linEqParamConstrs=[];                
        linIneqParamConstrs=[];   
        boundsOnParams=[-inf,-inf,1e-6;inf,inf,inf]
        
        nrOfNonlinParamIneqConstrs=0;        
        nrOfNonlinParamEqConstrs=0;          
        
        
        
    end
 
    methods
        %volume
        function v=vol(~,x) 
            v=pi*x(3)^2;
        end 
        function grad_v=gradVol(~,x)
           grad_v=[0;0;2*pi*x(3)]; 
        end
        
        %design functions
        function u=evalNonlinDescrFuns(~,x,y)
           u=(y(:,1)-x(1)).^2+(y(:,2)-x(2)).^2-x(3)^2; 
        end
        function Dy_u=DyNonlinDescrFuns(~,x,y) 
            Dy_u=[2*(y(1)-x(1)), 2*(y(2)-x(2))];
        end
        function Dx_u=DxNonlinDescrFuns(~,x,y)
            Dx_u=[-2*(y(1)-x(1)),-2*(y(2)-x(2)),-2*x(3)];
        end
        %Interior point
        function y=getInteriorPoint(~,x)
            y=[x(1),x(2)];
        end
        function Dx_y=DxInnerPoint(~,~)
            Dx_y=[eye(2),zeros(2,1)];
        end
        %bounding box
        function [lb,ub]=getBoundingBox(~,x)
            lb=[x(1)-x(3),x(2)-x(3)];
            ub=[x(1)+x(3),x(2)+x(3)];
        end
    end

    %transformation    
    methods
        function y=evalTrafoFun(obj,x,z)
            m=size(z,1);
            if m>0
                
                y=[x(1)+z(:,2).*x(3).*cos(z(:,1)), x(2)+z(:,2).*x(3).*sin(z(:,1))];
            else
                y=zeros(0,2);
            end 
        end
        function z=evalInvTrafoFun(obj,x,y)
            m=size(y,1);
            if m>0
                [THETA,RHO] = cart2pol(y(:,1)-x(:,1),y(:,2)-x(:,2));
                z = [THETA,RHO./(x(3))];
            else
                z=zeros(0,2);
            end
        end
        function Dx_y=DxTrafoFun(obj,x,z)
            m = size(z,1);
            o = ones(m,1);
            if m>0
                Dx_y = zeros(2,3,m);
                Dx_y(1,1,:) = o;
                Dx_y(2,2,:) = o;
                Dx_y(1,3,:) = z(:,2).*cos(z(:,1));
                Dx_y(2,3,:) = z(:,2).*sin(z(:,1));
            else
                Dx_y=[];
            end
        end
        function [lb,ub]=getBoxZ(obj)
            lb=[0,0];
            ub=[2*pi,1];
        end
        function detDz_y=detDzTrafoFun(obj,x,z)
            detDz_y=-x(3)^2*z(:,2);
        end
    end
    
    methods
        function []=plotDesign(obj,x)
            t = 0:pi/180:2*pi;
            o = ones(size(t));

            y1 = x(1)*o+x(3)*cos(t);
            y2 = x(2)*o+x(3)*sin(t);

            fill(y1,y2,'green');
            hold on;
        end
    end
    
    
end

