classdef Problem21 < ImplicitGSIPProblem
    % p=Problem21 creates an instance of the GSIP problem:
    %
    %       min   exp(x1)+exp(x2)+exp(x3)
    %
    %       s.t.  1/(1+y^2)-x1-x2*y-x3*y^2 <=0 for all y in Y(x)
    %             x2+x3 <= 0.5-10^-6
    %             -1 <= x1 <= 1
    %             -1 <= x2 <= 1
    %             -1 <= x3 <= 1    
    %
    %       where:
    %
    %       Y(x)={y in R with
    %               -0.5y<= -x2-x3
    %               0 <= y <= 1} 
    
    %One constraint is added, such that Y(x) doesn't become empty    
    properties(GetAccess=protected)
        
        nrOfUppLevVars=3;                          
        boundsOnUppLevVars=[-1,-1,-1;1,1,1];        
        nrOfSemiInfConstrs=1;                       
        initValsOfUppLevVars=[0,0,0];              
        
        linEqsOnUppLevVars=[];                     
        linIneqsOnUppLevVars=[0,1,1,1/2-1.e-6];          
        nrOfNonlinEqsUppLev=0;                
        nrOfNonlinIneqsUppLev=0;              
        
        
        nrOfLowLevVars=1;                          
        nrOfLowLevNonlinConstrs=0;                        
        
        con2LowLev=1;
        nrOfIndexset=1;
        
    end

    
    methods
        %objective function
        function f=evalObjFun(obj,x)                    
            f=exp(x(1))+exp(x(2))+exp(x(3));
        end
        function grad_f=gradObjFun(obj,x)                   
            grad_f=[exp(x(1));exp(x(2));exp(x(3))];
        end
        
        %upper level
        function g=evalSemiInfConstrs(obj,i,x,y)      
                                                         
            g=1./(1+y.^2)-x(1)-x(2)*y-x(3)*y.^2;            
        end
        function Dx_g=DxSemiInfConstrs(obj,i,x,y)     
            n=size(y,1);
            Dx_g=[-ones(n,1),-y,-y.^2];                
        end
        function Dy_g=DySemiInfConstrs(obj,i,x,y)
            Dy_g=-2*y./(1+y.^2).^2-x(2)-2*x(3)*y;                
        end
               
            
        function y=getInitValsLowLevVars(obj,~,x)           
            [lb,ub] = obj.getInterval(x);
            y=(lb+ub)/2;                
        end
        function Dx_y=DxStartingPointLowLev(obj,~,x)
            [Dx_lb,Dx_ub]=DxInterval(x);
            Dx_y=(Dx_lb+Dx_ub)/2;
        end
        
        function [lb,ub]=getBoundsOnLowLevVars(obj,~,x)    
             lb=-0;
             ub=1;
        end
                        
        %Linear inequalities
        function [A,b]=getLinIneqsOnLowLevVars(obj,k,x)
            A=[-0.5];
            b=[-x(2)-x(3)];
        end
        
        
        %transformation
        function y=evalTrafoFun(obj,i,x,z)
            [lb,ub]=obj.getInterval(x);
            y=lb*z+ub*(1-z);
        end
        function z=evalInvTrafoFun(obj,i,x,y)
            [lb,ub]=obj.getInterval(x);
            z=(y-ub)/((lb-ub));
                        
        end
        function Dx_y=DxTrafoFun(obj,i,x,z)
            [Dx_lb,Dx_ub]=obj.DxInterval(x);
            Dx_y=zeros(1,size(x,2),size(z,1));
            o=ones(size(z));
          
            Dx_y(1,:,:)=(o*Dx_lb*z+(1-z)*(Dx_ub))';
        end
        
   end
        methods(Access=private)
        
        function [lb,ub] = getInterval(obj,x)           
            lb=2*max(0,(x(2)+x(3)));
            ub=1;
        end
        function [Dx_lb,Dx_ub]=DxInterval(obj,x)
            if x(2)+x(3)>0
                Dx_lb=[0,2,2];
            else
                Dx_lb=[0,0,0];
            end
            Dx_ub=[0,0,0];
        end
        
    end
    
end
